package device.sdk.sample.button;

import android.app.Activity;
import android.os.Bundle;
import android.os.Environment;
import android.os.RemoteException;
import android.util.Log;
import android.text.Selection;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import device.common.HiJackData;
import device.common.HijackingKeys;
import device.sdk.KeyManager;

/** Simple Guide for PTT
 * 
 * import device.common.HiJackData;
 * import device.sdk.KeyManager;
 *  
 * int count = 0;
 * KeyManager keyManager = new KeyManager();
 * try {
 *		HiJackData[] dataList = keyManager.getAllHiJackData();
 * 		for (HiJackData data : dataList) {
 * 			switch (data.getDefaultKeyCode()) {
 * 				case 119: // KEYCODE_FUNCTION
 * 					data.setFlag(HiJackData.FLAG_UPDATE);
 * 					data.setConvertKeyCode("KEYCODE_EMERGENCY");
 * 					data.setConvertSymbol(269);
 * 					break;
 * 				case 260: // KEYCODE_SCANNER_R
 * 					data.setFlag(HiJackData.FLAG_UPDATE);
 * 					data.setConvertKeyCode("KEYCODE_PTT_R");
 * 					data.setConvertSymbol(270);
 * 					break;
 * 				case 261: // KEYCODE_SCANNER_L
 *  				data.setFlag(HiJackData.FLAG_UPDATE);
 *  				data.setConvertKeyCode("KEYCODE_PTT_L");
 *  				data.setConvertSymbol(271);
 *  				break;
 *  			defaults:
 *  				break;
 * 			}
 * 		}
 * 		count = getKeyManager().setAllHiJackData(dataList);
 * } catch (RemoteException e) {
 * 		e.printStackTrace();
 * } finally {
 * 		if (count > 0) {
 * 			Toast.makeText(this, "Complete to save", Toast.LENGTH_SHORT).show();
 * 		} else {
 * 			Toast.makeText(this, "toast_no_change", Toast.LENGTH_SHORT).show();
 * 		}
 * }
 * 
 */

public class ButtonSampleActivity extends Activity {
    private static final String TAG = ButtonSampleActivity.class.getSimpleName();
    
    private static final String KEYCODE_HOME_SYMBOL = "KEYCODE_HOME";
    private static final int KEYCODE_HOME_CODE = 3;
    
    private HiJackData[] mHiJackDataList;
    
    private TextView mPropertyLabel;
    private TextView mPropertySymbol;
    
    private TextView mDefineCurrent;
    private TextView mDefinePath;
    
    private KeyManager mKeyManager;    
    private KeyManager getKeyManager() {
    	if (mKeyManager == null) {
    		mKeyManager = new KeyManager();
    	}
    	return mKeyManager;
    }
    
    private void updatePropertyUI() {
        if (mHiJackDataList == null) {
            try {
                mHiJackDataList = getKeyManager().getAllHiJackData();
            } catch (RemoteException e) {
                e.printStackTrace();
            }
        }
        
        if (mPropertyLabel != null)
            mPropertyLabel.setText(mHiJackDataList[0].getLabel());
        if (mPropertySymbol != null)
            mPropertySymbol.setText(mHiJackDataList[0].getConvertSymbol());
        
        dumpHiJackData();
        dumpHijackingKeys();
    }
    
    private void changeProperty() {
        int count = 0;
        try {
            count = getKeyManager().setAllHiJackData(mHiJackDataList);
        } catch (RemoteException e) {
            e.printStackTrace();
        } finally {
            if (count > 0) {
                Toast.makeText(this, getString(R.string.toast_save_success), Toast.LENGTH_SHORT).show();
                updatePropertyUI();
            } else {
                Toast.makeText(this, getString(R.string.toast_no_change), Toast.LENGTH_SHORT).show();
            }
        }
    }
    
    private void updateDefineUI() {
        String currentKCMapFile = "";
        try {
            currentKCMapFile = getKeyManager().getCurrentKCMapFile();
        } catch (RemoteException e) {
            e.printStackTrace();
        }
        
        if (mDefineCurrent != null)
            mDefineCurrent.setText(currentKCMapFile);
        if (mDefinePath != null) {
            mDefinePath.setText(Environment.getExternalStorageDirectory() + "/");
            Selection.setSelection(mDefinePath.getEditableText(), mDefinePath.length());
        }
    }
    
    private void changeDefine() {
        int result = 0;
        try {
            result = getKeyManager().changeKCMapFile(mDefinePath.getText().toString());
        } catch (RemoteException e) {
            e.printStackTrace();
        } finally {            
            if (result == 0) {
                Toast.makeText(this, getString(R.string.toast_save_success), Toast.LENGTH_SHORT).show();
                updateDefineUI();                
            } else {
                String failMsg = getString(R.string.toast_save_fail) + "(" + result + ")";
                Toast.makeText(this, failMsg, Toast.LENGTH_SHORT).show();
            }
        }
    }
    
    private void defaultDefine() {
        boolean result = true;
        try {
            result = getKeyManager().removeKCMapFile();
        } catch (RemoteException e) {
            e.printStackTrace();
        } finally {            
            if (result) {
                Toast.makeText(this, getString(R.string.toast_save_success), Toast.LENGTH_SHORT).show();
                updateDefineUI();                
            } else {
                Toast.makeText(this, getString(R.string.toast_save_fail), Toast.LENGTH_SHORT).show();
            }
        }
    }
    
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_button_sample);
        
        mPropertyLabel = (TextView) findViewById(R.id.label_properies);
        mPropertySymbol = (TextView) findViewById(R.id.symbol_properies);
        
        updatePropertyUI();
        
        Button saveProperty = (Button) findViewById(R.id.save_properties);
        saveProperty.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                if (mHiJackDataList == null) return;
                mHiJackDataList[0].setFlag(HiJackData.FLAG_UPDATE);
                mHiJackDataList[0].setConvertKeyCode(KEYCODE_HOME_CODE);
                mHiJackDataList[0].setConvertSymbol(KEYCODE_HOME_SYMBOL);
                changeProperty();
            }
        });
        Button defaultProperty = (Button) findViewById(R.id.default_properties);
        defaultProperty.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                if (mHiJackDataList == null) return;
                mHiJackDataList[0].setFlag(HiJackData.FLAG_UPDATE);
                mHiJackDataList[0].setConvertKeyCode(mHiJackDataList[0].getDefineKeyCode());
                mHiJackDataList[0].setConvertSymbol(mHiJackDataList[0].getDefineSymbol());
                changeProperty();
            }
        });
        
        mDefineCurrent = (TextView) findViewById(R.id.current_define);
        mDefinePath = (TextView) findViewById(R.id.path_define);
        
        updateDefineUI();
        
        Button saveDefine = (Button) findViewById(R.id.save_define);
        saveDefine.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                if (mDefinePath == null) return;
                changeDefine();
            }
        });
        Button defaultDefine = (Button) findViewById(R.id.default_define);
        defaultDefine.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                defaultDefine();
            }
        });
        
        dumpHiJackData();
        dumpHijackingKeys();
    }
    
    private void dumpHiJackData() {
    	Log.e(TAG, "These are the stored data to the database... Start dump >>>");
    	HiJackData[] hiJackDataList = null;
    	try {
    		hiJackDataList = getKeyManager().getAllHiJackData();
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    	if (hiJackDataList != null) {
    		for (HiJackData data : hiJackDataList) {
    			Log.e(TAG, "ID(" + data.getID()
    			+ "), Label(" + data.getLabel()
    			+ "), DefaultKey(" + data.getDefaultSymbol() + "-" + data.getDefineKeyCode()
    			+ "), ConvertKey(" + data.getConvertSymbol() + "-" + data.getConvertKeyCode() + ")");
    		}
    	}
    	Log.e(TAG, "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ End dump <<<");
    }
    
    private void dumpHijackingKeys() {
    	Log.e(TAG, "These are the defined key list... Start dump >>>");
    	HijackingKeys[] hijackingKeys = null;
    	try {
    		hijackingKeys = getKeyManager().getHijackingKeys();
    	}
    	catch (RemoteException e) {
            e.printStackTrace();
        }
    	if (hijackingKeys != null) {
    		for (HijackingKeys data : hijackingKeys) {
    			Log.e(TAG, "Label(" + data.getLabel()
    			+ "), DefaultKey(" + data.getDefaultSymbol() + "-" + data.getDefineKeyCode() + ")");
    		}
    	}
    	Log.e(TAG, "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ End dump <<<");
    }
}
