package device.sdk.sample.mdm;

import android.content.Intent;
import android.content.res.Configuration;
import android.os.Bundle;
import android.preference.ListPreference;
import android.preference.Preference;
import android.preference.PreferenceFragment;
import android.view.MenuItem;

import device.sdk.Control;
import device.sdk.sample.mdm.MdmSampleActivity;
import device.sdk.sample.mdm.R;
import device.sdk.sample.mdm.tool.DropDownPreference;
import device.sdk.sample.mdm.tool.SeekBarPreference;

public class DisplayPreferenceFragment extends PreferenceFragment implements Preference.OnPreferenceChangeListener {
    private static final String KEY_BRIGHTNESS_LEVEL = "brightness_level";
    private static final String KEY_AUTO_ROTATE = "auto_rotate";
    private static final String KEY_SCREEN_TIMEOUT = "screen_timeout";
    private SeekBarPreference mBrightnessLevel;
    private ListPreference mScreenTimeout;
    private DropDownPreference mAutoRotate;
    private int mMaxBrightnessLevel = 0;

    private static Control mControl = new Control();

    private void bindPreferenceChange() {
            /* KEY_BRIGHTNESS_LEVEL */
        mBrightnessLevel = (SeekBarPreference) findPreference(KEY_BRIGHTNESS_LEVEL);
        mMaxBrightnessLevel = mControl.getScreenBrightnessMax();
        mBrightnessLevel.setMax(mMaxBrightnessLevel);
        mBrightnessLevel.setProgress(mControl.getScreenBrightness());
        mBrightnessLevel.setOnPreferenceChangeListener(this);
        onPreferenceChange(mBrightnessLevel, mBrightnessLevel.getProgress());
            /* KEY_AUTO_ROTATE */
        mAutoRotate = (DropDownPreference) findPreference(KEY_AUTO_ROTATE);
        if (mControl.isRotationLockToggleVisible()) {
            mAutoRotate.addItem(getString(R.string.pref_auto_rotate_rotate), false);
            int rotateLockedResourceId;
            if (mControl.areAllRotationsAllowed()) {
                rotateLockedResourceId = R.string.pref_auto_rotate_stay_in_current;
            } else {
                if (mControl.getRotationLockOrientation() == Configuration.ORIENTATION_PORTRAIT) {
                    rotateLockedResourceId = R.string.pref_auto_rotate_stay_in_portrait;
                } else {
                    rotateLockedResourceId = R.string.pref_auto_rotate_stay_in_landscape;
                }
            }
            mAutoRotate.addItem(getString(rotateLockedResourceId), true);
            mAutoRotate.setSelectedItem(mControl.isRotationLocked() ? 1 : 0);
            mAutoRotate.setCallback(new DropDownPreference.Callback() {
                @Override
                public boolean onItemSelected(int pos, Object value) {
                    final boolean locked = (Boolean) value;
                    mControl.setRotationLock(locked);
                    return true;
                }
            });
        } else {
            mAutoRotate.setEnabled(false);
        }
            /* KEY_SCREEN_TIMEOUT */
        mScreenTimeout = (ListPreference) findPreference(KEY_SCREEN_TIMEOUT);
        mScreenTimeout.setValue(String.valueOf(mControl.getScreenOffTimeout()));
        mScreenTimeout.setOnPreferenceChangeListener(this);
        onPreferenceChange(mScreenTimeout, mScreenTimeout.getValue());
    }

    @Override
    public boolean onPreferenceChange(Preference preference, Object value) {
        boolean result = true;
        final String key = preference.getKey();
        if (KEY_BRIGHTNESS_LEVEL.equals(key)) {
            mControl.setScreenBrightness(false, (Integer) value);
            mBrightnessLevel.setTitle(getString(R.string.pref_title_brightness_level, (Integer) value, mMaxBrightnessLevel));
        } else if (KEY_SCREEN_TIMEOUT.equals(key)) {
            int currentTimeout = Integer.parseInt((String) value);
            mControl.setScreenOffTimeout(currentTimeout);
            if (currentTimeout < -1) {
                mScreenTimeout.setSummary("");
            } else {
                final CharSequence[] entries = mScreenTimeout.getEntries();
                final CharSequence[] values = mScreenTimeout.getEntryValues();
                if (entries == null || entries.length == 0) {
                    mScreenTimeout.setSummary("");
                } else if(currentTimeout == -1){
                    mScreenTimeout.setSummary(entries[values.length-1].toString());
                } else {
                    int best = 0;
                    for (int i = 0; i < values.length; i++) {
                        long timeout = Long.parseLong(values[i].toString());
                        if (currentTimeout >= timeout && (timeout > 0)) { best = i; }
                    }
                    mScreenTimeout.setSummary(getString(R.string.pref_summary_screen_timeout, entries[best]));
                }
            }
        }
        return result;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        addPreferencesFromResource(R.xml.pref_display);
        setHasOptionsMenu(true);
        ((MdmSampleActivity) getActivity()).setupActionBar(true);
    }

    @Override
    public void onResume() {
        super.onResume();
        bindPreferenceChange();
    }

    @Override
    public void onPause() {
        super.onPause();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();
        if (id == android.R.id.home) {
            startActivity(new Intent(getActivity(), MdmSampleActivity.class));
            return true;
        }
        return super.onOptionsItemSelected(item);
    }
}