package device.sdk.sample.mdm;

import android.content.Intent;
import android.os.Bundle;
import android.os.UserManager;
import android.preference.Preference;
import android.preference.PreferenceFragment;
import android.preference.SwitchPreference;
import android.view.MenuItem;

import device.sdk.Control;
import device.sdk.sample.mdm.MdmSampleActivity;
import device.sdk.sample.mdm.R;

public class SecurityPreferenceFragment extends PreferenceFragment implements Preference.OnPreferenceChangeListener {
    private static final String KEY_ENABLED_SCREEN_CAPTURE = "enabled_screen_capture";
    private static final String KEY_LOCK_UNKNOWN_SOURCES_OPTION = "lock_unknown_sources_option";
    private static final String KEY_UNKNOWN_SOURCES = "unknown_sources";
    private static final String KEY_DISALLOW_UNKNOWN_SOURCES = "disallow_unknown_sources";
    private SwitchPreference mEnabledScreenCapture;
    private SwitchPreference mLockUnknownSourcesOption;
    private SwitchPreference mUnknownSources;
    private SwitchPreference mDisallowUnknownSources;

    private static Control mControl = new Control();

    private void bindPreferenceChange() {
            /* KEY_ENABLED_SCREEN_CAPTURE */
        mEnabledScreenCapture = (SwitchPreference) findPreference(KEY_ENABLED_SCREEN_CAPTURE);
        mEnabledScreenCapture.setChecked(mControl.isEnabledScreenCapture());
        mEnabledScreenCapture.setOnPreferenceChangeListener(this);
        onPreferenceChange(mEnabledScreenCapture, mEnabledScreenCapture.isChecked());
            /* KEY_UNKNOWN_SOURCES */
        mUnknownSources = (SwitchPreference) findPreference(KEY_UNKNOWN_SOURCES);
        mUnknownSources.setChecked(mControl.isNonMarketAppsAllowed());
        mUnknownSources.setOnPreferenceChangeListener(this);
        onPreferenceChange(mUnknownSources, mUnknownSources.isChecked());
            /* KEY_DISALLOW_UNKNOWN_SOURCES */
        mDisallowUnknownSources = (SwitchPreference) findPreference(KEY_DISALLOW_UNKNOWN_SOURCES);
        mDisallowUnknownSources.setChecked(mControl.hasUserRestriction(UserManager.DISALLOW_INSTALL_UNKNOWN_SOURCES));
        mDisallowUnknownSources.setOnPreferenceChangeListener(this);
        onPreferenceChange(mDisallowUnknownSources, mDisallowUnknownSources.isChecked());
            /* KEY_LOCK_UNKNOWN_SOURCES_OPTION */
        mLockUnknownSourcesOption = (SwitchPreference) findPreference(KEY_LOCK_UNKNOWN_SOURCES_OPTION);
        mLockUnknownSourcesOption.setChecked(mControl.isNonMarketAppsAllowedOption());
        mLockUnknownSourcesOption.setOnPreferenceChangeListener(this);
        onPreferenceChange(mLockUnknownSourcesOption, mLockUnknownSourcesOption.isChecked());
    }

    @Override
    public boolean onPreferenceChange(Preference preference, Object value) {
        boolean result = true;
        final String key = preference.getKey();
        if (KEY_ENABLED_SCREEN_CAPTURE.equals(key)) {
            mControl.setEnabledScreenCapture((Boolean) value);
            mEnabledScreenCapture.setSummary(getString(R.string.pref_summary_boolean_param, (Boolean) value));
        } else if (KEY_LOCK_UNKNOWN_SOURCES_OPTION.equals(key)) {
            mControl.setNonMarketAppsAllowedOption((Boolean) value);
            mLockUnknownSourcesOption.setSummary(getString(R.string.pref_summary_boolean_param, (Boolean) value));
            boolean enable = mControl.isNonMarketAppsAllowedOption() && !mControl.isRestricted() &&
                    !mControl.hasUserRestriction(UserManager.DISALLOW_INSTALL_APPS);
            mUnknownSources.setEnabled(enable);
        } else if (KEY_UNKNOWN_SOURCES.equals(key)) {
            mControl.setNonMarketAppsAllowed((Boolean) value);
            mUnknownSources.setSummary(getString(R.string.pref_summary_boolean_param, (Boolean) value));
        } else if (KEY_DISALLOW_UNKNOWN_SOURCES.equals(key)) {
            mControl.setUserRestriction(UserManager.DISALLOW_INSTALL_UNKNOWN_SOURCES, (Boolean) value);
            mDisallowUnknownSources.setSummary(getString(R.string.pref_summary_boolean_param, (Boolean) value));
            boolean enable = mControl.isNonMarketAppsAllowedOption() && !mControl.isRestricted() &&
                    !mControl.hasUserRestriction(UserManager.DISALLOW_INSTALL_APPS);
            mUnknownSources.setEnabled(enable);
        }
        return result;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        addPreferencesFromResource(R.xml.pref_security);
        setHasOptionsMenu(true);
        ((MdmSampleActivity) getActivity()).setupActionBar(true);
    }

    @Override
    public void onResume() {
        super.onResume();
        bindPreferenceChange();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();
        if (id == android.R.id.home) {
            startActivity(new Intent(getActivity(), MdmSampleActivity.class));
            return true;
        }
        return super.onOptionsItemSelected(item);
    }
}