package com.example.vender.preview;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Matrix;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Environment;
import android.os.RemoteException;
import android.os.Handler;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.Toast;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.nio.ByteBuffer;
import java.util.Calendar;
import java.util.Arrays;

import device.common.ScanConst;
import device.sdk.ScanManager;

public class tPreviewActivity extends Activity {
	public static final String TAG = "tPreviewActivity";

    private static final int PREVIEW_SOURCE_WIDTH = 416;
    private static final int PREVIEW_SOURCE_HEIGHT = 320;
    private static final int RAW_DATA_SIZE = PREVIEW_SOURCE_WIDTH * PREVIEW_SOURCE_HEIGHT;

	private static byte[] mPart = new byte[100000];
	private static byte[] mBuffer = new byte[RAW_DATA_SIZE * 4];

	private static ScanManager mScanner = null;

	private static boolean mKeyLock = false;
	private static Preview mPreview = null;
	private static Button mButton = null;
	private static StreamReadTask mReadTask = null;

    private static Context mContext = null;
    private AlertDialog mDialog = null;

    private ProgressDialog mWaitDialog = null;
    private final Handler mHandler = new Handler();

	private BroadcastReceiver mScanKeyEventReceiver = new BroadcastReceiver() {
		@Override
		public void onReceive(Context context, Intent intent) {
			if (ScanConst.INTENT_SCANKEY_EVENT.equals(intent.getAction())) {
				KeyEvent event = (KeyEvent) intent.getParcelableExtra(ScanConst.EXTRA_SCANKEY_EVENT);
				switch (event.getKeyCode()) {
                    case 276:
                    case 277:
                    case 278:
                    case 290:
					case ScanConst.KEYCODE_SCAN_FRONT:
					case ScanConst.KEYCODE_SCAN_LEFT:
					case ScanConst.KEYCODE_SCAN_RIGHT:
					case ScanConst.KEYCODE_SCAN_REAR:
						if (mScanner != null) {
							if (event.getAction() == KeyEvent.ACTION_DOWN && !mKeyLock) {
								previewStart();
							} else if (event.getAction() == KeyEvent.ACTION_UP) {
								previewStop();
                                try {Thread.sleep(100);} catch (InterruptedException e) {}
								capture();
							}
						}
						break;
				}
			}
		}
	};

	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.preview_activity);
        Log.e(TAG, "onCreate()");

        mContext = this;
        mScanner = new ScanManager();

		mPreview = (Preview) findViewById(R.id.preview);
		mButton = (Button) findViewById(R.id.PreviewCaptureButton);
		mButton.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View view) {
                if (mScanner != null) {
                    if (mKeyLock) {
                        previewStop();
                        try {Thread.sleep(100);} catch (InterruptedException e) {}
                        capture();
                    } else {
                        previewStart();
                    }
                }
            }
        });
	}

	@Override
	protected void onDestroy() {
		if (mScanner != null) {
			mScanner.aDecodeSetScanImageMode(ScanConst.ScanMode.DCD_MODE_SCAN);
            mScanner = null;
		}
        super.onDestroy();
	}

    @Override
    protected void onResume() {
        super.onResume();
		mWaitDialog = ProgressDialog.show(mContext, "", getString(R.string.msg_wait), true);
		try {
			Thread.sleep(500);
		} catch (InterruptedException e) {}

		if (mPreview != null) {
			mPreview.resumePreview();
		}
        if (mScanner != null) {
            if (mScanner.aDecodeGetDecodeEnable() == 1) {
                if (getEnableDialog().isShowing()) {
                    getEnableDialog().dismiss();
                }
                mScanner.aDecodeSetScanImageMode(ScanConst.ScanMode.DCD_MODE_IMAGE);
            } else {
                if (!getEnableDialog().isShowing()) {
                    getEnableDialog().show();
                }
            }
        }
        registerReceiver(mScanKeyEventReceiver, new IntentFilter(ScanConst.INTENT_SCANKEY_EVENT));
        mHandler.postDelayed(mStartOnResume, 1000);
    }

    @Override
    protected void onPause() {
        mWaitDialog = null;
        unregisterReceiver(mScanKeyEventReceiver);
        if (mScanner != null) {
            if (mKeyLock) {
                previewStop();
            }
            mScanner.aDecodeSetScanImageMode(ScanConst.ScanMode.DCD_MODE_SCAN);
        }
		if (mPreview != null) {
			mPreview.pausePreview();
		}
        mHandler.removeCallbacks(mStartOnResume);
        super.onPause();
    }

    private Runnable mStartOnResume = new Runnable() {
        @Override
        public void run() {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    if (mWaitDialog != null) {
                        mWaitDialog.dismiss();
                    }
                }
            });
        }
    };

    private AlertDialog getEnableDialog() {
        if (mDialog == null) {
            final AlertDialog dialog = new AlertDialog.Builder(this).create();
            dialog.setTitle(R.string.app_name);
            dialog.setMessage("Your scanner is disabled. Do you want to enable it?");
            dialog.setButton(AlertDialog.BUTTON_NEGATIVE, getString(android.R.string.cancel),
                    new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialogInterface, int i) {
                            finish();
                        }
                    });
            dialog.setButton(AlertDialog.BUTTON_POSITIVE, getString(android.R.string.ok),
                    new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialogInterface, int i) {
                            Intent intent = new Intent(ScanConst.LAUNCH_SCAN_SETTING_ACITON);
                            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                            startActivity(intent);
                            dialog.dismiss();
                        }
                    });
            dialog.setCancelable(false);
            mDialog = dialog;
        }
        return mDialog;
    }

	private static void previewStart() {
		if (mScanner != null) {
            mScanner.aDecodeImageStreamInit();
            mScanner.aDecodeImageStreamStart();
            mReadTask = new StreamReadTask();
            mReadTask.execute();
            mButton.setText("Stop/Capture");
            mKeyLock = true;
		}
	}

	private static void previewStop() {
		if (mReadTask != null) {
			mReadTask.stop();
            mReadTask = null;
		}
		if (mScanner != null) {
            mScanner.aDecodeImageStreamStop();
		}
		mButton.setText("Preview");
        mKeyLock = false;
	}

	private static void capture() {
		if (mScanner != null) {
            int size = 0;
            int pos = 0;
            Arrays.fill(mBuffer, (byte) 0);
			do {
				Arrays.fill(mPart, (byte) 0);
				size = mScanner.aDecodeImageCapture(mPart);
				Log.d(TAG, "Partial size : " + size);
				System.arraycopy(mPart, 0, mBuffer, pos, size);
				Log.d(TAG, "Buffer position : " + pos);
				pos += size - 1;
			} while (mPart.length <= size);

            if (pos > 0) {
                saveCapture(mBuffer);
            }
        }
	}

	private static ByteBuffer makeBuffer(byte[] src) {
        byte[] bits = new byte[src.length*4];
        for (int i = 0; i < src.length; i++) {
            bits[i * 4] = (byte) (src[i]);
            bits[i * 4 + 1] = (byte) (src[i]);
            bits[i * 4 + 2] = (byte) (src[i]);
            bits[i * 4 + 3] = (byte) 0xff;
        }
        return ByteBuffer.wrap(bits);
    }

    private static int getDate() {
        int data = 0;
        Calendar cal = Calendar.getInstance();
        data = cal.get(Calendar.YEAR) * 10000;
        data += (cal.get(Calendar.MONTH)+1) * 100;
        data += cal.get(Calendar.DAY_OF_MONTH);
        return data;
    }

    private static int getTime() {
        int time = 0;
        Calendar cal = Calendar.getInstance();
        time = cal.get(Calendar.HOUR_OF_DAY) * 10000;
        time += cal.get(Calendar.MINUTE) * 100;
        time += cal.get(Calendar.SECOND);
        return time;
    }

    private static void saveCapture(byte[] buffer) {
        String folderPath;
        String savePath;
        Bitmap captureImage;
        folderPath = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_PICTURES).getAbsolutePath()
                + "/" + mContext.getString(R.string.app_name);
        File folder = new File(folderPath);
        if (!folder.exists()) {
            folder.mkdir();
        }
        savePath = folderPath + "/" + getDate() + "_" + getTime() + ".jpeg";
        try {
            FileOutputStream out = new FileOutputStream(savePath);
            if (PREVIEW_SOURCE_WIDTH == 0 || PREVIEW_SOURCE_HEIGHT == 0) {
                captureImage = BitmapFactory.decodeByteArray(buffer, 0, buffer.length);
                Matrix matrix = new Matrix();
                matrix.postRotate(0);
                captureImage = Bitmap.createBitmap(captureImage, 0, 0, captureImage.getWidth(), captureImage.getHeight(), matrix, false);
            } else {
                captureImage = Bitmap.createBitmap(PREVIEW_SOURCE_WIDTH * 2, PREVIEW_SOURCE_HEIGHT * 2, Bitmap.Config.ARGB_8888);
                captureImage.copyPixelsFromBuffer(makeBuffer(buffer));
            }
            captureImage.compress(Bitmap.CompressFormat.JPEG, 100, out);
            startFileMediaScan(mContext, savePath);
        } catch (FileNotFoundException e) {}
    }

    private static void startFileMediaScan(Context context, String path){
        if (context != null) {
            context.sendBroadcast(new Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE, Uri.parse("file://" + path)));
        }
    }

	private static class StreamReadTask extends AsyncTask<Void, Void, Void> {
		byte[] mBuffer = new byte[RAW_DATA_SIZE];
		int mSize = 0;
		boolean mStop = false;

		public void stop() {
			mPreview.stopPreview();
			mStop = true;
		}

		@Override
		protected Void doInBackground(Void... arg0) {
			while (!mStop) {
                mSize = mScanner.aDecodeImageStreamRead(mBuffer);
                if (mSize != 0) {
                    mPreview.setPreview(mBuffer, PREVIEW_SOURCE_WIDTH, PREVIEW_SOURCE_HEIGHT);
                }
				try {Thread.sleep(25);} catch (InterruptedException e) {}
			}
			return null;
		}
	}
}
